import requests  # برای ارسال درخواست HTTP به API
import pandas as pd  # برای کار با داده‌ها و ذخیره در اکسل
import schedule  # برای زمان‌بندی اجرای خودکار کد
import time  # برای کنترل اجرای متناوب اسکریپت
from datetime import datetime  # برای دریافت تاریخ و زمان فعلی

# ------------------- دریافت قیمت لحظه‌ای بیت‌کوین -------------------
def get_btc_price():
    url = "https://api.coingecko.com/api/v3/simple/price?ids=bitcoin&vs_currencies=usd"  # آدرس API برای دریافت قیمت بیت‌کوین به دلار
    response = requests.get(url)  # ارسال درخواست به API
    if response.status_code == 200:  # بررسی موفقیت‌آمیز بودن دریافت داده
        data = response.json()  # تبدیل پاسخ به فرمت JSON
        return data["bitcoin"]["usd"]  # بازگرداندن قیمت بیت‌کوین به دلار
    else:
        return None  # بازگرداندن مقدار None در صورت بروز خطا

# ------------------- ذخیره قیمت در فایل اکسل -------------------
def save_price_to_excel():
    price = get_btc_price()  # دریافت قیمت بیت‌کوین
    if price is None:
        print("❌ خطا در دریافت قیمت بیت‌کوین!")  # نمایش خطا در صورت عدم دریافت قیمت
        return

    # خواندن تاریخ و زمان ثبت قیمت
    now_date = datetime.now().strftime("%Y-%m-%d")  # دریافت تاریخ فعلی به فرمت YYYY-MM-DD
    now_time = datetime.now().strftime("%H:%M:%S")  # دریافت ساعت فعلی به فرمت HH:MM:SS

    # ایجاد DataFrame جدید با اطلاعات جدید
    new_data = pd.DataFrame([[now_date, now_time, price]], columns=["Date", "Time", "BTC Price"])

    # نام فایل اکسل برای ذخیره داده‌ها
    file_name = "btc_price_history.xlsx"

    try:
        # تلاش برای خواندن فایل اکسل موجود
        existing_data = pd.read_excel(file_name)
        updated_data = pd.concat([existing_data, new_data], ignore_index=True)  # اضافه کردن داده جدید به داده‌های قبلی
    except FileNotFoundError:
        # اگر فایل وجود نداشت، داده جدید را ایجاد کن
        updated_data = new_data

    # ذخیره داده‌های جدید در فایل اکسل
    updated_data.to_excel(file_name, index=False)
    print(f"✅ قیمت بیت‌کوین ({price} USD) در {now_date} - {now_time} ذخیره شد!")  # پیام موفقیت ذخیره داده

# ------------------- اجرای خودکار هر ۶ ساعت -------------------
schedule.every(6).hours.do(save_price_to_excel)  # تنظیم اجرا هر ۶ ساعت

# اجرای مداوم برای بررسی زمان اجرا و اجرای وظایف زمان‌بندی‌شده
while True:
    schedule.run_pending()  # بررسی زمان‌بندی اجرا
    time.sleep(60)  # بررسی هر دقیقه برای اجرای وظایف
